/* Copyright 1999 by Donald J. Bindner */

#include <bios.h>
#include <conio.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "input.h"
#include "convert.h"

/* indexing to make for help that is properly alphabetized */
int hl[] = { 0, 1, 24, 2, 25, 26, 27, 28, 29, 30, 31, 32, 3, 4,
                    33, 34, 5, 35, 36, 37, 6, 38, 39, 40, 41, 43, 42, 7,
                    44, 45, 46, 8, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56,
                    57, 58, 9, 10, 59, 60, 11, 61, 62, 63, 12, 64, 65, 66,
                    67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 79, 80, 78,
                    81, 82, 13, 83, 84, 85, 87, 88, 86, 14, 15, 89, 90, 91,
                    92, 93, 94, 95, 16, 96, 97, 98, 99, 100, 17, 101, 102,
                    103, 104, 18, 19, 105, 20, 106, 107, 108, 109, 110, 111,
                    112, 21, 22, 23, 113, 114, 115, 116, 117, 118, 119, 120 };

#define BLACK 0
#define LGRAY 7
#define LCYAN 11
#define WHITE 15
#define CYAN 3
#define RED 4
#ifndef PI
  #define PI 3.14159265359
#endif
#define FT3_PER_BU 1.24445601263

#define MENU_LENGTH 8
#define MENU_FIRST 0
#define MENU_LAST (MENU_LENGTH-1)

void centerline( char *message, int fore, int back );
void ecb1();
void bcw();
void ddays();
void yield();
void cvol();
void rvol();
void conv();
void quit();

extern UNIT unit[];

static char *digits = "0123456789";
static char *decimal = "0123456789.";
static char *neg_decimal = "-0123456789.";
static char *letters = "^*/0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

main()
{
    char *menu[MENU_LENGTH] = {
        " 1st European corn borer ",
        " Black cutworm           ",
        " Growing degree days     ",
        " Corn yield estimate     ",
        " Circular volumes        ",
        " Rectangular volumes     ",
        " Unit conversions        ",
        " Quit                    "
    };

    void (*func[])() = {
        ecb1, bcw, ddays, yield, cvol, rvol, conv, quit
    };

    int highlighted, i;
    int ch;

    highlighted = 0;
    printmenu:

    clrscrn();
    cursoff();

    poscurs( 3, 0 );
    centerline( "Farm utilities by Donald Bindner", CYAN, BLACK );

    for( i = MENU_FIRST; i < MENU_LENGTH; i++ ) {
        poscurs( 5+i, 0 );
        if( i == highlighted ) {
            centerline( menu[i], WHITE, RED );
        } else {
            centerline( menu[i], LGRAY, BLACK );
        }
    }
    while( 1 ) {
        ch = getch();
        if( ch == 13 ) {
            ( *func[ highlighted ] )();
            goto printmenu;
        } else if( ch == 27 ) {
            if( highlighted == MENU_LAST ) {
                ungetch( 13 );
            } else {
                poscurs( 5+highlighted, 0 );
                centerline( menu[highlighted], LGRAY, BLACK );
                highlighted = MENU_LAST;
                poscurs( 5+highlighted, 0 );
                centerline( menu[highlighted], WHITE, RED );
            }
        } else if( !ch ) {
            ch = getch();
            poscurs( 5+highlighted, 0 );
            centerline( menu[highlighted], LGRAY, BLACK );

            switch( ch ) {
                case 80:
                case 77: highlighted = (highlighted+1) % MENU_LENGTH;
                         break;
                case 72:
                case 75: highlighted = (highlighted+MENU_LENGTH-1) % MENU_LENGTH;
                         break;

                case 73:
                case 71: highlighted = MENU_FIRST;
                         break;
                case 81:
                case 79: highlighted = MENU_LAST;
                         break;
            }

            poscurs( 5+highlighted, 0 );
            centerline( menu[ highlighted ], WHITE, RED );
        }
    }
}

void blank( int last ) {
    writechs( ' ', LGRAY + BLACK*15, last-curscol());
}

void clreol() {
    writechs( ' ', LGRAY + BLACK*16, 80-curscol());
}

void centerline( char *message, int fore, int back ) {
    int attr;

    poscurs( cursrow(), (80 - strlen( message ))/2 );
    attr = fore + back*16;

    while( *message ) {
        writechs( *message, attr, 1 );
        message++;
        poscurs( cursrow(), curscol()+1 );
    }
}

void title( char *message ) {
    poscurs( 3, 0 );
    centerline( message, CYAN, BLACK );
}

void ecb1() {
    static int field_size[3] = { 3, 4, 3 };
    char *field_mask[3] = { digits, decimal, digits };
    static double field_val[3] = { 100.0, 1.30, 5.0 };
    static char inp_field[3][5] = { "100", "1.30", "15" };
    int fields_entered = 0, line = 5, action;

    clrscrn();

    title( "1st European corn borer" );

    poscurs( line++, 0 );
    centerline( "   Potential yield:     bu/ac " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "        Corn price:      $/bu " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( " Per cent infested:     %     " , LCYAN, BLACK );

    poscurs( ++line, 0 );
    centerline( "Gain for treatment:       $/ac" , LCYAN, BLACK );

    line = 5;
    do {
        poscurs( line, 45 );
        action = input( inp_field[line-5], field_size[line-5], field_mask[line-5], BLACK, LGRAY );
        field_val[line-5] = atof( inp_field[ line-5 ] );
        if( fields_entered < line-4 ) fields_entered++;

        poscurs( line, 44 );
        switch( line ) {
            case 5: printf( "%4.0f ", field_val[0] );
                    break;
            case 6: printf( "%5.2f ", field_val[1] );
                    break;
            case 7: printf( "%4.0f ", field_val[2] );
                    break;
        }

        if( fields_entered == 3 ) {
            poscurs( 9, 44 );
            printf( "% 3.2f ", field_val[0] * field_val[1] * field_val[2] *
                              0.8 * 0.0008 );
        }

        switch( action ) {
            case 13:                                    /* enter */
            case 9:                                     /* tab */
            case 80*256: line = ( line - 4 )%3 + 5;     /* down arrow */
                         break;
            case 15*256:                                /* shift tab */
            case 72*256: line = ( line - 3 )%3 + 5;     /* up arrow */
                         break;
        }
    } while( action != 27 );
}

void bcw() {
    static int field_size[4] = { 3, 3, 3, 3 };
    char *field_mask[4] = { digits, digits, digits, decimal };
    static double field_val[4] = { 65.0, 80.0, 60.0, 2.0 };
    static char inp_field[4][4] = { "65", "80", "60", "2.0" };
    double dd, table;
    int fields_entered = 0, line = 5, action;

    clrscrn();

    title( "Black cutworm activity for replanting" );

    poscurs( line++, 0 );
    centerline( "       Soil temperature:     F   " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "Maximum air temperature:     F   " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "Minimum air temperature:     F   " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "     Head capsule width:     mm   " , LCYAN, BLACK );
    line++;

    poscurs( line++, 0 );
    centerline( " Days to corn emergence:      days" , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "  Degree days (base 50):      days" , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "    Time until pupation:      days" , LCYAN, BLACK );

    line = 5;
    do {
        poscurs( line, 48 );
        action = input( inp_field[line-5], field_size[line-5], field_mask[line-5], BLACK, LGRAY );
        field_val[line-5] = atof( inp_field[ line-5 ] );
        if( fields_entered < line-4 ) fields_entered++;

        poscurs( line, 47 );
        switch( line ) {
            case 5: printf( "%4.0f ", field_val[0] );
                    break;
            case 6: printf( "%4.0f ", field_val[1] );
                    break;
            case 7: printf( "%4.0f ", field_val[2] );
                    break;
            case 8: printf( "%4.1f ", field_val[3] );
                    break;
        }

        if( fields_entered == 4 ) {
            dd = ( field_val[1] + field_val[2] ) / 2.0 - 50.0;

            if( field_val[3] < 1.5 ) {            /* some kind of table lookup */
                table = 3.5;
            } else if( field_val[3] < 2.0 ) {
                table = 4.5;
            } else if( field_val[3] < 3.0 ) {
                table = 5.5;
            } else if( field_val[3] < 4.0 ) {
                table = 6.5;
            } else table = 7.5;

            poscurs( 10, 47 );
            printf( "%5.1f ", 4.0 + 50.0*( field_val[0]/80.0 - 80.0/field_val[0] )
                                       * log( field_val[0]/80.0 ));
            poscurs( 11, 47 );
            printf( "%5.1f ", dd );
            poscurs( 12, 47 );
            printf( "%5.1f ", ( 750.0 - table*110.0 + 170.0 ) / dd );
        }

        switch( action ) {
            case 13:                                    /* enter */
            case 9:                                     /* tab */
            case 80*256: line = ( line - 4 )%4 + 5;     /* down arrow */
                         break;
            case 15*256:                                /* shift tab */
            case 72*256: line = ( line - 2 )%4 + 5;     /* up arrow */
                         break;
        }
    } while( action != 27 );
}

void ddays() {
    static int field_size[2] = { 3, 3 };
    char *field_mask[2] = { digits, digits };
    static double field_val[2] = { 60.0, 80.0 };
    static char inp_field[2][4] = { "60", "80" };
    int fields_entered = 0, line = 5, action;

    clrscrn();
    title( "Growing degree days / base 50" );

    poscurs( line++, 0 );
    centerline( "Maximum air temperature:     F   ", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "Minimum air temperature:     F   ", LCYAN, BLACK );

    poscurs( ++line, 0 );
    centerline( "            Degree days:      days", LCYAN, BLACK );

    line = 5;
    do {
        poscurs( line, 48 );
        action = input( inp_field[line-5], field_size[line-5], field_mask[line-5], BLACK, LGRAY );
        field_val[line-5] = atof( inp_field[ line-5 ] );
        if( fields_entered < line-4 ) fields_entered++;

        poscurs( line, 47 );
        printf( "%4.0f ", field_val[line-5] );

        if( fields_entered == 2 ) {
            poscurs( 8, 47 );
            printf( "%5.1f ", ( field_val[0] + field_val[1] ) / 2.0 - 50.0 );
        }

        switch( action ) {
            case 13:                                    /* enter */
            case 9:                                     /* tab */
            case 15*256:                                /* shift tab */
            case 72*256:                                /* up arrow */
            case 80*256: line = ( line - 4 )%2 + 5;     /* down arrow */
                         break;
        }
    } while( action != 27 );
}

void yield() {
    static int field_size[5] = { 4, 5, 3, 4, 4 };
    char *field_mask[5] = { decimal, decimal, digits, decimal, decimal };
    static double field_val[5] = { 30.0, 17.4, 20.0, 16.0, 40.0 };
    static char inp_field[5][6] = { "30.0", "17.4", "20", "16.0", "40.0" };
    double weight, pop;
    int fields_entered = 0, line = 5, action;

    clrscrn();
    title( "Corn yield estimate" );

    poscurs( line++, 0 );
    centerline( "      Field row spacing:      in.       ", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "         Length sampled:       ft.      ", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "           Ears counted:     ears       ", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "Rows of kernals per ear:      rows      ", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "        Kernals per row:      ker       ", LCYAN, BLACK );
    line++;

    poscurs( line++, 0 );
    centerline( "        Estimated yield:       bu/ac    ", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "       Field population:         ears/ac", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "   Estimated ear weight:      lb/ear    ", LCYAN, BLACK );

    line = 5;
    do {
        poscurs( line, 45 );
        action = input( inp_field[line-5], field_size[line-5], field_mask[line-5], BLACK, LGRAY );
        field_val[line-5] = atof( inp_field[ line-5 ] );
        if( fields_entered < line-4 ) fields_entered++;

        poscurs( line, 44 );
        switch( line ) {
            case 5: printf( "%5.1f ", field_val[0] );
                    break;
            case 6: printf( "%6.1f ", field_val[1] );
                    break;
            case 7: printf( "%4.0f ", field_val[2] );
                    break;
            case 8: printf( "%5.1f ", field_val[3] );
                    break;
            case 9: printf( "%5.1f ", field_val[4] );
                    break;
        }

        if( fields_entered == 5 ) {
            pop = 43560.0 * field_val[2] * 12.0 / field_val[0] / field_val[1];
            weight = field_val[3] * field_val[4] * 0.00078;

            poscurs( 11, 44 );
            printf( "%6.1f ", pop * weight / 70.0 );
            poscurs( 12, 44 );
            printf( "%8.1f ", pop );
            poscurs( 13, 44 );
            printf( "%5.2f ", weight );
        }

        switch( action ) {
            case 13:                                    /* enter */
            case 9:                                     /* tab */
            case 80*256: line = ( line - 4 )%5 + 5;     /* down arrow */
                         break;
            case 15*256:                                /* shift tab */
            case 72*256: line = ( line - 1 )%5 + 5;     /* up arrow */
                         break;
        }
    } while( action != 27 );
}

void cvol() {
    static int field_size[3] = { 5, 5, 5 };
    char *field_mask[3] = { decimal, decimal, neg_decimal };
    static double field_val[3] = { 30.0, 40.0 };
    static char inp_field[3][6] = { "30.0", "40.0", "5.0" };
    int fields_entered = 0, line = 5, action;
    double base_area;

    clrscrn();
    title( "Circular bin volume" );

    poscurs( line++, 0 );
    centerline( "   Diameter of the bin:       ft. ", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "Height to base of cone:       ft. ", LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "        Height of cone:       ft. ", LCYAN, BLACK );

    poscurs( ++line, 0 );
    centerline( "               Bushels:         bu", LCYAN, BLACK );

    line = 5;

    do {
        poscurs( line, 47 );
        action = input( inp_field[line-5], field_size[line-5], field_mask[line-5], BLACK, LGRAY );
        field_val[line-5] = atof( inp_field[ line-5 ] );
        if( fields_entered < line-4 ) fields_entered++;

        poscurs( line, 46 );
        printf( "%6.1f ", field_val[line-5] );

        if( fields_entered == 3 ) {
            base_area = PI*( field_val[0]/2.0 )*( field_val[0]/2.0 );
            poscurs( 9, 46 );
            printf( "%8.1f ", base_area*( field_val[1] + field_val[2]/3.0 )
                                      / FT3_PER_BU );
        }

        switch( action ) {
            case 13:                                    /* enter */
            case 9:                                     /* tab */
            case 80*256: line = ( line - 4 )%3 + 5;     /* down arrow */
                         break;
            case 15*256:                                /* shift tab */
            case 72*256: line = ( line - 3 )%3 + 5;     /* up arrow */
                         break;
        }
    } while( action != 27 );
}

void rvol() {
    static int field_size[4] = { 5, 5, 5, 5 };
    char *field_mask[4] = { decimal, decimal, decimal, neg_decimal };
    static double field_val[4] = { 65.0, 80.0, 14.0, 2.0 };
    static char inp_field[4][6] = { "65.0", "80.0", "14.0", "2.0" };
    int fields_entered = 0, line = 5, action;
    double base_area;

    clrscrn();

    title( "Rectangular storage volumes" );

    poscurs( line++, 0 );
    centerline( "        Length:       ft  " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "         Width:       ft  " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "        Height:       ft  " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "Height of cone:       ft  " , LCYAN, BLACK );

    poscurs( ++line, 0 );
    centerline( "       Bushels:         bu" , LCYAN, BLACK );

    line = 5;
    do {
        poscurs( line, 43 );
        action = input( inp_field[line-5], field_size[line-5], field_mask[line-5], BLACK, LGRAY );
        field_val[line-5] = atof( inp_field[ line-5 ] );
        if( fields_entered < line-4 ) fields_entered++;

        poscurs( line, 42 );
        printf( "%6.1f ", field_val[line-5] );

        if( fields_entered == 4 ) {
            base_area = field_val[0] * field_val[1];

            poscurs( 10, 42 );
            printf( "%8.1f ", base_area*( field_val[2] + field_val[3]/3.0 )
                                      / FT3_PER_BU );
        }

        switch( action ) {
            case 13:                                    /* enter */
            case 9:                                     /* tab */
            case 80*256: line = ( line - 4 )%4 + 5;     /* down arrow */
                         break;
            case 15*256:                                /* shift tab */
            case 72*256: line = ( line - 2 )%4 + 5;     /* up arrow */
                         break;
        }
    } while( action != 27 );
}

void unit_menu( void ) {
    int i, j, ch, attr, old, redraw = 0, redraw_all = 1;
    static int line=0, pos=0;
    char text[121];
    static char *format = " %-6s  %-27s  %-20s  ";

    poscurs( 13, 0 );
    writechs( '', CYAN + BLACK*16, 1 );
    poscurs( 13, 1);
    writechs( '', CYAN + BLACK*16, 78 );
    poscurs( 13, 79 );
    writechs( '', CYAN + BLACK*16, 1 );

    for( i = 14; i < 24; i++ ) {
        poscurs( i, 0 );
        writechs( '', CYAN + BLACK*16, 1 );
        poscurs( i, 79 );
        writechs( '', CYAN + BLACK*16, 1 );
    }

    poscurs( 24, 0 );
    writechs( '', CYAN + BLACK*16, 1 );
    poscurs( 24, 1);
    writechs( '', CYAN + BLACK*16, 78 );
    poscurs( 24, 79 );
    writechs( '', CYAN + BLACK*16, 1 );

    while( 1 ) {
        if( redraw ) {
            sprintf( text, format, unit[hl[old]].name,
                           unit[hl[old]].fname, unit[hl[old]].desc );
            strcat( text, unit_string( hl[old] ));
            attr = LCYAN + BLACK*16;
            for( j = 0; text[j]; j++ ) {
                poscurs( 14+old-line, j+1 );
                writechs( text[j], attr, 1 );
            }
            poscurs( 14+old-line, j+1 );

            sprintf( text, format, unit[hl[pos]].name,
                           unit[hl[pos]].fname, unit[hl[pos]].desc );
            strcat( text, unit_string( hl[pos] ));
            attr = RED + BLACK*16;
            for( j = 0; text[j]; j++ ) {
                poscurs( 14+pos-line, j+1 );
                writechs( text[j], attr, 1 );
            }
            poscurs( 14+pos-line, j+1 );
            redraw = 0;
        } else if( redraw_all ) for( i = 0; i < 10; i++ ) {
            sprintf( text, format, unit[hl[i+line]].name,
                           unit[hl[i+line]].fname, unit[hl[i+line]].desc );
            strcat( text, unit_string( hl[i+line] ));
            if( i+line == pos ) {
                attr = RED + BLACK*16;
            } else {
                attr = LCYAN + BLACK*16;
            }

            for( j = 0; text[j]; j++ ) {
                poscurs( i+14, j+1 );
                writechs( text[j], attr, 1 );
            }
            poscurs( i+14, j+1 );
            blank( 79 );
        }
        redraw_all = 0;

        ch = getch();

        if( !ch && kbhit()) {
            ch = getch();
            switch( ch ) {
                case 72: if( pos > 0 ) {           /* up */
                            old = pos;
                            pos--;
                            if( pos < line ) {
                                line -= 9;
                                if( line < 0 ) line = 0;
                                redraw_all = 1;
                            } else {
                                redraw = 1;
                            }
                        }
                        break;
                case 80: if( pos < NUM_UNITS-1 ) {   /* down */
                            old = pos;
                            pos++;
                            if( pos > line+9 ) {
                                line += 9;
                                if( line > NUM_UNITS - 10 ) line = NUM_UNITS - 10;
                                redraw_all = 1;
                            } else {
                                redraw = 1;
                            }
                        }
                        break;
                case 73: if( line > 0 ) {            /* pgup */
                            line -= 9;
                            if( line < 0 ) line = 0;

                            pos -= 9;
                            if( pos < 0 ) pos = 0;
                            redraw_all = 1;
                        }
                        break;
                case 81: if( line < NUM_UNITS - 10 ) {      /* pgdn */
                            line += 9;
                            if( line > NUM_UNITS - 10 ) line = NUM_UNITS - 10;
                            pos+= 9;
                            if( pos > NUM_UNITS - 1 ) pos = NUM_UNITS - 1;
                            redraw_all = 1;
                        }
                        break;
                case 71: if( line > 0 ) {                  /* home */
                            line = pos = 0;
                            redraw_all = 1;
                        } else if( pos > 0 ) {
                            old = pos;
                            pos = 0;
                            redraw = 1;
                        }
                        break;
                case 79: if( line < NUM_UNITS - 10 ) {      /* end */
                            line = NUM_UNITS - 10;
                            pos = NUM_UNITS - 1;
                            redraw_all = 1;
                        } else if( pos < NUM_UNITS - 1 ) {
                            old = pos;
                            pos = NUM_UNITS - 1;
                            redraw = 1;
                        }
                        break;
            }
        } else if( ch == 27) {
            break;
        }
    }

    for( i = 13; i < 25; i++ ) {
        poscurs( i, 0 );
        clreol();
    }
}

void conv() {
    static int field_size[3] = { 11, 15, 15 };
    char *field_mask[3] = { "-0123456789.eE", letters, letters };
    static double field_val = 10.0;
    static char inp_field[3][16] = { "10.0", "m/s", "mi/h" };
    int fields_entered = 0, line = 5, action;
    double conversion;

    clrscrn();

    title( "Unit conversions adapted from the HP28S" );

    poscurs( line++, 0 );
    centerline( "Amount to convert:              " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "     Convert from:              " , LCYAN, BLACK );
    poscurs( line++, 0 );
    centerline( "       Convert to:              " , LCYAN, BLACK );

    poscurs( ++line, 0 );
    centerline( "    Equivalent to:              " , LCYAN, BLACK );

    poscurs( line += 2, 0 );
    centerline( "F1: Help  F2:    F3:          " , CYAN, BLACK );

    poscurs( 9, 45 );       /* clean up the color of this field */
    clreol();
 
    line = 5;
    do {
        poscurs( line, 43 );
        action = input( inp_field[line-5], field_size[line-5], field_mask[line-5], BLACK, LGRAY );
        if( line == 5 ) field_val = atof( inp_field[ line-5 ] );
        if( fields_entered < line-4 ) fields_entered++;

        poscurs( line, 43 );
        switch( line ) {
            case 5: printf( "%11.8f", field_val );
                    clreol();
                    break;
            case 6: printf( "%s", inp_field[1] );
                    clreol();
                    break;
            case 7: printf( "%s", inp_field[2] );
                    clreol();
                    break;
        }

        if( fields_entered == 3 ) {
            poscurs( 9, 45 );
            conversion = field_val;
            if( !convert( inp_field[1], inp_field[2], &conversion )) {
                printf( "Couldn't convert  check units" );
                clreol();
            } else {
                printf( "%.8g %s", conversion, inp_field[2] );
                clreol();
            }
        }

        switch( action ) {
            case 13:                                    /* enter */
            case 9:                                     /* tab */
            case 80*256: line = ( line - 4 )%3 + 5;     /* down arrow */
                         break;
            case 15*256:                                /* shift tab */
            case 72*256: line = ( line - 3 )%3 + 5;     /* up arrow */
                         break;
            case 59*256: unit_menu();
                         break;
        }
    } while( action != 27 );
}

void quit() {
    curson();
    exit( 0 );
}
