/* Copyright 1999 by Donald J. Bindner */

#include <bios.h>
#include <conio.h>
#include <string.h>
#include "input.h"

#ifndef min
 #define min(a,b)    (((a) < (b)) ? (a) : (b))
#endif

int input( char *message, int size, char *mask, int fore, int back ) {
    int i, crow, ccol, attr, pos, vlen, r_code = 0, key_hit = 0;
    int ch;

    crow = cursrow();
    ccol = curscol();
    attr = fore + back*16;
    message[size] = 0;
    vlen = pos = min( strlen( message ), size );

    for( i = vlen; i < size; i++ ) {
        message[i] = ' ';
    }

    for( i = 0; i < size; i++ ) {
        poscurs( crow, ccol+i );
        writechs( message[i], attr, 1 );
    }

    do {
        poscurs( crow, ccol + pos );
        curson();
    jump_into_loop:
        ch = getch();
        cursoff();

        if( ch && ( strchr( mask, (int)ch ) != NULL )) {
            if( !key_hit ) {
                for( i = vlen-1; i >= 0; i-- ) {
                    poscurs( crow, ccol + i );
                    writechs( ' ', attr, 1 );
                    message[i] = ' ';
                }
                vlen = pos = 0;
            }

            if( pos < size ) {
                message[pos] = (char)ch;
                writechs( ch, attr, 1 );
                pos++;
                if( pos > vlen ) vlen = pos;
            }
        } else if( ch == 8 ) {         /* backspace */
            if( pos > 0 ) {
                for( i = pos; i < vlen; i++ ) {
                    message[i-1] = message[i];
                    poscurs( crow, ccol + i - 1 );
                    writechs( message[i-1], attr, 1 );
                }
                message[--vlen] = ' ';
                poscurs( crow, ccol + vlen );
                writechs( ' ', attr, 1 );
                pos--;
            }
        } else if( ch && ( ch < 32 )) {
            r_code = (int)ch;         /* line-feed */
        } else if(( !ch ) && kbhit() ) {
            ch = getch();
            switch(ch) {
                case 15:                         /* back-tab */
                case 72:                         /* up */
                case 73:                         /* pgup */
                case 80:                         /* down */
                case 59:
                case 81: r_code = (int)ch * 256; /* pgdn */
                         break;
                case 75: if( pos > 0 ) pos--;    /* left */
                         break;
                case 77: if( pos < vlen ) pos++; /* right */
                         break;
                case 71: pos = 0;                /* home */
                         break;
                case 79: pos = vlen;             /* end */
                         break;
                case 83: if( pos < vlen ) {
                             for( i = pos+1; i < vlen; i++ ) {
                                 message[i-1] = message[i];
                                 poscurs( crow, ccol + i - 1 );
                                 writechs( message[i-1], attr, 1 );
                             }
                             message[--vlen] = ' ';
                             poscurs( crow, ccol + vlen );
                             writechs( ' ', attr, 1 );
                         }
                         break;
                case 60: if( strchr( mask, '' ) != NULL ) {
                             ungetch( '' );  /* cheap way to add  */
                             goto jump_into_loop;
                         }
                         break;
                case 61: if( strchr( mask, '' ) != NULL ) {
                             ungetch( '' );  /* cheap way to add  */
                             goto jump_into_loop;
                         }
                         break;
            }
        }
        if( !key_hit ) key_hit++;
    } while( !r_code );

    i = size;                                    /* trim trailing space */
    while(( i > 0 ) && ( message[i-1] == ' ' )) i--;
    message[i] = 0;

    return r_code;
}


